@@ -33,15 +33,13 @@
 #include <cstdlib>    // For atoi()
 #include <sys/stat.h> // For stat()
 #include <algorithm> // For transform()
+#include <unistd.h>  // For read() close()
 
 #ifdef WIN32
 #include <io.h>
-#define pfsize(st) (st.st_size)
 #define close _close
 #define read _read
 #define SHUT_RDWR SD_BOTH
-#else
-#define pfsize(st) (st.st_blksize * st.st_blocks)
 #endif
 
 #include "url.hpp"
@@ -55,13 +53,8 @@ using namespace std;
 
 // This is the maximum pac size (to avoid memory attacks)
 #define PAC_MAX_SIZE 102400
-
-const string url::GENERIC_DELIMITERS(":/?#[]@");
-const string url::SUBCOMPONENT_DELIMITERS("!$&'()*+,;=");
-const string url::ALLOWED_IN_USERINFO_ELEMENT(url::SUBCOMPONENT_DELIMITERS);
-const string url::ALLOWED_IN_USERINFO(url::ALLOWED_IN_USERINFO_ELEMENT + ":");
-const string url::ALLOWED_IN_PATH_ELEMENT(url::SUBCOMPONENT_DELIMITERS + ":@");
-const string url::ALLOWED_IN_PATH(url::ALLOWED_IN_PATH_ELEMENT + "/");
+// This is the default block size to use when receiving via HTTP
+#define PAC_HTTP_BLOCK_SIZE 512
 
 static inline int get_default_port(string scheme) {
 	struct servent *serv;
@@ -104,13 +97,13 @@ bool url::is_valid(const string url_) {
 		rtv = false;
 	}
 
-    return rtv;
+	return rtv;
 }
 
 string url::encode(const string &data, const string &valid_reserved) {
 	ostringstream encoded;
-	for (int i=0; data[i]; i++) {
-		if (isalnum(data[i])
+	for (unsigned int i=0; i < data.size(); i++) {
+		if (isalnum((unsigned char)data[i])
 				|| valid_reserved.find(data[i]) != string::npos
 				|| string("-._~").find(data[i]) != string::npos)
 			encoded << data[i];
@@ -126,6 +119,7 @@ url::url(const string &url) throw(parse_error)
 	: m_orig(url), m_port(0), m_ips(NULL) {
 	size_t idx = 0;
 	size_t hier_part_start, hier_part_end;
+	size_t query_part_start;
 	size_t path_start, path_end;
 	string hier_part;
 
@@ -158,9 +152,17 @@ url::url(const string &url) throw(parse_error)
 	transform(m_scheme.begin(), m_scheme.end(), m_scheme.begin(), ::tolower);
 
 	hier_part_start = idx;
-	hier_part_end = url.find('?', idx);
-	if (hier_part_end == string::npos)
-		hier_part_end = url.find('#', idx);
+	hier_part_end = url.find('#', idx);
+	query_part_start = url.find('?', idx);
+	if (query_part_start != string::npos)
+	{
+		if (hier_part_end == string::npos)
+			m_query = url.substr(query_part_start);
+		else {
+			m_query = url.substr(query_part_start, hier_part_end - query_part_start);
+		}
+		hier_part_end = query_part_start;
+	}
 
 	hier_part = url.substr(hier_part_start,
 							hier_part_end == string::npos ?
@@ -181,8 +183,8 @@ url::url(const string &url) throw(parse_error)
 		authority_start = 2;
 		/* authority is always followed by / or nothing */
 		authority_end = hier_part.find('/', authority_start);
-                if (authority_end == string::npos)
-                  authority_end = hier_part.size();
+		if (authority_end == string::npos)
+			authority_end = hier_part.size();
 		path_start = authority_end;
 
 		/* 3.2:
@@ -211,7 +213,8 @@ url::url(const string &url) throw(parse_error)
 			host_start = userinfo_end + 1;
 
 		/* Check for IPv6 IP */
-		if (hier_part[host_start] == '[') {
+		if (host_start < hier_part.size()
+				&& hier_part[host_start] == '[') {
 			host_end = hier_part.find(']', host_start);
 			if (host_end == string::npos)
 				throw parse_error("Invalid URL: " + url);
@@ -220,6 +223,8 @@ url::url(const string &url) throw(parse_error)
 				host_end = string::npos;
 		} else {
 			host_end = hier_part.find(':', host_start);
+			if (path_start < host_end)
+				host_end = path_start;
 		}
 
 		/* If not port, host ends where path starts */
@@ -232,7 +237,7 @@ url::url(const string &url) throw(parse_error)
 		/* Get port */
 		m_port = get_default_port(m_scheme);
 
-		if (host_end != hier_part.size()
+		if (host_end < hier_part.size()
 			&& hier_part[host_end] == ':') {
 			size_t port_start = host_end + 1;
 			m_port = atoi(hier_part.c_str() + port_start);
@@ -273,6 +278,7 @@ url& url::operator=(const url& url) {
 	m_orig   = url.m_orig;
 	m_pass   = url.m_pass;
 	m_path   = url.m_path;
+	m_query  = url.m_query;
 	m_port   = url.m_port;
 	m_scheme = url.m_scheme;
 	m_user   = url.m_user;
@@ -360,6 +366,10 @@ string url::get_path() const {
 	return m_path;
 }
 
+string url::get_query() const {
+	return m_query;
+}
+
 uint16_t url::get_port() const {
 	return m_port;
 }
@@ -390,7 +400,7 @@ static inline string recvline(int fd) {
 
 char* url::get_pac() {
 	int sock = -1;
-	bool correct_mime_type = false, chunked = false;
+	bool chunked = false;
 	unsigned long int content_length = 0, status = 0;
 	char* buffer = NULL;
 	string request;
@@ -400,13 +410,16 @@ char* url::get_pac() {
 		struct stat st;
 		if ((sock = ::open(m_path.c_str(), O_RDONLY)) < 0)
 			return NULL;
-		if (!fstat(sock, &st) && pfsize(st) < PAC_MAX_SIZE) {
-			buffer = new char[pfsize(st)+1];
-			if (read(sock, buffer, pfsize(st)) == 0) {
-				delete buffer;
+
+		if (!fstat(sock, &st) && st.st_size < PAC_MAX_SIZE) {
+			buffer = new char[st.st_size+1];
+			memset(buffer, 0, st.st_size+1);
+			if (read(sock, buffer, st.st_size) == 0) {
+				delete[] buffer;
 				buffer = NULL;
 			}
 		}
+		close(sock);
 		return buffer;
 	}
 
@@ -435,7 +448,7 @@ char* url::get_pac() {
 	if (sock < 0) return NULL;
 
 	// Build the request string
-	request  = "GET " + m_path + " HTTP/1.1\r\n";
+	request  = "GET " + (m_path.size() > 0 ? m_path : "/") + m_query + " HTTP/1.1\r\n";
 	request += "Host: " + m_host + "\r\n";
 	request += "Accept: " + string(PAC_MIME_TYPE) + "\r\n";
 	request += "Connection: close\r\n";
@@ -451,15 +464,10 @@ char* url::get_pac() {
 	string line = recvline(sock);
 	if (sscanf(line.c_str(), "HTTP/1.%*d %lu", &status) == 1 && status == 200) {
 		/* Check for correct mime type and content length */
+		content_length = 0;
 		for (line = recvline(sock) ; line != "\r" && line != "" ; line = recvline(sock)) {
-			// Check for content type
-			if (line.find("Content-Type: ") == 0 &&
-				(line.find(PAC_MIME_TYPE) != string::npos ||
-				 line.find(PAC_MIME_TYPE_FB) != string::npos))
-				correct_mime_type = true;
-
 			// Check for chunked encoding
-			else if (line.find("Content-Transfer-Encoding: chunked") == 0)
+			if (line.find("Content-Transfer-Encoding: chunked") == 0 || line.find("Transfer-Encoding: chunked") == 0)
 				chunked = true;
 
 			// Check for content length
@@ -470,7 +478,7 @@ char* url::get_pac() {
 		// Get content
 		unsigned int recvd = 0;
 		buffer = new char[PAC_MAX_SIZE];
-		*buffer = '\0';
+		memset(buffer, 0, PAC_MAX_SIZE);
 		do {
 			unsigned int chunk_length;
 
@@ -484,25 +492,31 @@ char* url::get_pac() {
 				// Add this chunk to our content length,
 				// ensuring that we aren't over our max size
 				content_length += chunk_length;
-				if (content_length >= PAC_MAX_SIZE) break;
 			}
 
-			while (recvd != content_length) {
-				int r = recv(sock, buffer + recvd, content_length - recvd, 0);
-				if (r < 0) break;
+			if (content_length >= PAC_MAX_SIZE) break;
+
+			while (content_length == 0 || recvd != content_length) {
+				int r = recv(sock, buffer + recvd,
+				             content_length == 0 ? PAC_HTTP_BLOCK_SIZE
+				                                 : content_length - recvd, 0);
+				if (r <= 0) {
+					chunked = false;
+					break;
+				}
 				recvd += r;
 			}
-			buffer[content_length] = '\0';
 		} while (chunked);
 
-		if (string(buffer).size() != content_length) {
-			delete buffer;
+		if (content_length != 0 && string(buffer).size() != content_length) {
+			delete[] buffer;
 			buffer = NULL;
 		}
 	}
 
 	// Clean up
 	shutdown(sock, SHUT_RDWR);
+	close(sock);
 	return buffer;
 }
 
